import java.io.*;
import java.awt.image.BufferedImage;
import javax.imageio.*;
import java.util.Scanner;

public class PPMImage {

    private int width;
    private int height;
    private RGB[][] pixels;

    public PPMImage() {}

    public PPMImage(PPMImage copy) {
        this.width = copy.width;
        this.height = copy.height;
        this.pixels = new RGB[this.width][this.height];

        for (int i=0; i < this.width; i++) {
            for (int j=0; j < this.height; j++) {
                this.pixels[i][j] = new RGB(
                    copy.pixels[i][j].R,
                    copy.pixels[i][j].G,
                    copy.pixels[i][j].B
                );
            }
        }
    }

    /* Reads in a PPM file into a PPMImage. */
    public static PPMImage readPPM(String fileName) {
        PPMImage img = new PPMImage();

        /* Read in the type of PPM.  We ignore this. */

        /* Read in the height and width. */

        /* Read in the maximum size each color.  This should always be 255. */

        /* Initialize the array of pixels. */

        /* For each pixel we read in, create a new RGB. */
        
        throw new NotImplementedException("readPPM is not implemented.");
    }

    /* Reads in a PNG file into a PPMImage. */
    public static PPMImage readPNG(String fileName) {
        PPMImage newImg = new PPMImage();

        try {
            BufferedImage img = ImageIO.read(new File(fileName));
            newImg.width = img.getWidth();
            newImg.height = img.getHeight();

            newImg.pixels = new RGB[newImg.width][newImg.height];

            for (int i=0; i < newImg.width; i++) {
                for (int j=0; j < newImg.height; j++) {
                    int rgb = img.getRGB(i,j);
                    char r = (char)((rgb >> 16) & 0x000000FF);
                    char g = (char)((rgb >>  8) & 0x000000FF);
                    char b = (char)((rgb >>  0) & 0x000000FF);
                    newImg.pixels[i][j] = new RGB(r, g, b);
                }
            }
            return newImg;
        }
        catch (IOException e) {
            return null;
        }
    }

    /* Writes out a PPMImage into a file.
     * @returns true on success, false on failure */
    public boolean writePPM(String fileName) {
        throw new NotImplementedException("writePPM is not implemented.");

        /* Create a new file and a new writer. */

        /* Write out the PPM type */

        /* Write out the height and width. */

        /* Write out the maximum color. */
        
        /* Write out each pixel. */

        /* Don't forget to close the file. */
    }

    /* Writes out a PPMImage as a PNG. */ 
    public boolean writePNG(String fileName) {
        BufferedImage img = new BufferedImage(this.width, this.height, BufferedImage.TYPE_INT_RGB);
        File out = new File(fileName);

        for (int i=0; i < this.width; i++) {
            for (int j=0; j < this.height; j++) {
                RGB rgb = this.pixels[i][j];
                int r = (rgb.R << 16) & 0x00FF0000;
                int g = (rgb.G <<  8) & 0x0000FF00;
                int b = (rgb.B <<  0) & 0x000000FF;
                img.setRGB(i, j, r | g | b);
            }
        }
        try {
            ImageIO.write(img, "png", out);
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    public int getHeight() {
        return this.height;
    }

    public int getWidth() {
        return this.width;
    }

    public String getPPMType() {
        return "P3";
    }

    public String getHeightAndWidth() {
        return Integer.toString(this.height) + " " + Integer.toString(this.width);
    }

    public RGB getPixel(int x, int y) {
        return this.pixels[x][y];
    }

    public void setPixel(int x, int y, RGB rgb) {
        this.pixels[x][y] = rgb;
    }
    
    public static void main(String[] args) {
        /* Do a round trip for some image. */
        String filename = "monty_hall";
        PPMImage png = PPMImage.readPNG(filename + ".png");
        if (png == null) {
            System.out.printf("Hmm...is %s.png a real image?\n", filename);
            System.exit(1);
        }
        if (png.writePPM(filename + ".ppm")) {
            PPMImage.readPPM(filename + ".ppm").writePNG(filename + ".png");
        }
        else {
            System.out.printf("Uh oh.  Something went wrong when writing %s.ppm\n", filename);
            System.exit(1);
        }
    }
}
